"""
Module with implementation of colors and their formatter functions.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from mighty_logger.basic.exceptions import ColorException
from mighty_logger.src.ansi_format import GetAnsiFormat

ColorPicker = {
	# Color table
	# Original: https://en.wikipedia.org/wiki/Web_colors
	# Red
	'MAROON': [128, 0, 0],
	'DARKRED': [139, 0, 0],
	'RED': [255, 0, 0],
	'LIGHTRED': [255, 51, 51],
	'FIREBRICK': [178, 34, 34],
	'CRIMSON': [220, 20, 60],
	'INDIANRED': [205, 92, 92],
	'LIGHTCORAL': [240, 158, 128],
	'SALMON': [250, 128, 114],
	'DARKSALMON': [233, 150, 122],
	'LIGHTSALMON': [255, 160, 122],
	# Pink
	'MEDIUMVIOLETRED': [199, 21, 133],
	'DEEPPINK': [255, 20, 147],
	'PALEVIOLETRED': [219, 112, 147],
	'HOTPINK': [255, 105, 180],
	'LIGHTPINK': [255, 182, 193],
	'PINK': [255, 192, 203],
	# Orange
	'ORANGERED': [255, 69, 0],
	'TOMATO': [255, 99, 71],
	'DARKORANGE': [255, 140, 0],
	'CORAL': [255, 127, 80],
	'ORANGE': [255, 165, 0],
	# Yellow
	'DARKKHAKI': [189, 183, 107],
	'GOLD': [255, 215, 0],
	'KHAKI': [240, 230, 140],
	'PEACHPUFF': [255, 218, 185],
	'YELLOW': [255, 255, 0],
	'DARKYELLOW': [255, 204, 0],
	'PALEGOLDENROD': [238, 232, 170],
	'MOCCASIN': [255, 228, 181],
	# Purple
	'INDIGO': [75, 0, 130],
	'PURPLE': [128, 0, 128],
	'DARKMAGENTA': [139, 0, 139],
	'DARKVIOLET': [148, 0, 211],
	'DARKSLATEBLUE': [82, 61, 139],
	'BLUEVIOLET': [138, 43, 226],
	'DARKORCHID': [153, 50, 204],
	'FUCHSIA': [255, 0, 255],
	'SLATEBLUE': [106, 90, 205],
	'MEDIUMSLATEBLUE': [127, 104, 238],
	'MEDIUMORCHID': [186, 85, 211],
	'MEDIUMPURPLE': [147, 112, 219],
	'ORCHID': [218, 112, 214],
	'VIOLET': [238, 130, 238],
	'PLUM': [221, 160, 221],
	'THISTLE': [216, 191, 216],
	'LAVENDER': [230, 230, 250],
	# Green
	'DARKGREEN': [0, 100, 0],
	'GREEN': [0, 128, 0],
	'DARKOLIVEGREEN': [85, 107, 47],
	'FORESTGREEN': [34, 139, 34],
	'SEAGREEN': [46, 139, 87],
	'DARKSLATEGRAY': [47, 79, 79],
	'OLIVE': [128, 128, 0],
	'OLIVEDRAB': [107, 142, 35],
	'MEDIUMSEAGREEN': [60, 179, 113],
	'LIMEGREEN': [50, 205, 50],
	'LIME': [0, 255, 0],
	'SPRINGGREEN': [0, 255, 127],
	'MEDIUMSPRINGGREEN': [0, 250, 154],
	'DARKSEAGREEN': [143, 188, 143],
	'MEDIUMAQUAMARINE': [102, 205, 170],
	'YELLOWGREEN': [154, 205, 50],
	'LAWNGREEN': [124, 252, 0],
	'CHARTREUSE': [127, 255, 0],
	'LIGHTGREEN': [144, 238, 144],
	'GREENYELLOW': [173, 255, 47],
	'PALEGREEN': [152, 251, 152],
	# Aqua
	'TEAL': [0, 128, 128],
	'DARKCYAN': [0, 139, 139],
	'LIGHTSEAGREEN': [32, 178, 170],
	'CADETBLUE': [95, 158, 160],
	'DARKTURQUOISE': [0, 206, 209],
	'MEDIUMTURQUOISE': [72, 209, 204],
	'TURQUOISE': [64, 224, 208],
	'AQUA': [0, 255, 255],
	'AQUAMARINE': [127, 255, 212],
	'SKYBLUE': [135, 206, 235],
	'LIGHTSKYBLUE': [135, 206, 250],
	'LIGHTSTEELBLUE': [176, 196, 222],
	'LIGHTBLUE': [173, 216, 230],
	'POWDERBLUE': [176, 224, 230],
	'PALETURQUOISE': [175, 238, 238],
	# Blue
	'MIDNIGHTBLUE': [25, 25, 112],
	'NAVY': [0, 0, 128],
	'DARKBLUE': [0, 0, 139],
	'MEDIUMBLUE': [0, 0, 205],
	'BLUE': [0, 0, 255],
	'ROYALBLUE': [65, 105, 225],
	'STEELBLUE': [90, 130, 180],
	'DODGERBLUE': [30, 144, 255],
	'DEEPSKYBLUE': [0, 191, 255],
	'CORNFLOWERBLUE': [100, 149, 237],
	# Brown
	'BROWN': [165, 42, 42],
	'SADDLEBROWN': [139, 69, 19],
	'SIENNA': [160, 82, 45],
	'CHOCOLATE': [210, 105, 30],
	'DARKGOLDENROD': [184, 134, 11],
	'PERU': [205, 133, 63],
	'ROSYBROWN': [188, 143, 143],
	'GOLDENROD': [218, 165, 32],
	'SANDYBROWN': [244, 164, 96],
	'TAN': [210, 180, 140],
	'BURLYWOOD': [222, 184, 135],
	'WHEAT': [245, 222, 179],
	'NAVAJOWHITE': [255, 222, 173],
	'BISQUE': [255, 228, 196],
	'BLANCHEDALMOND': [255, 235, 205],
	# White
	'WHITE': [255, 255, 255],
	'SNOW': [255, 250, 250],
	'HONEYDEW': [240, 255, 240],
	'MINTCREAM': [245, 255, 250],
	'AZURE': [240, 255, 255],
	'LIGHTCYAN': [224, 255, 255],
	'ALICEBLUE': [240, 248, 255],
	'GHOSTWHITE': [248, 248, 255],
	'WHITESMOKE': [245, 245, 245],
	'SEASHELL': [255, 245, 238],
	'BEIGE': [245, 245, 220],
	'OLDLACE': [253, 245, 230],
	'FLORALWHITE': [255, 250, 240],
	'IVORY': [255, 255, 240],
	'ANTIQUEWHITE': [250, 235, 215],
	'LINEN': [250, 240, 230],
	'LAVENDERBLUSH': [255, 240, 245],
	'MISTYROSE': [255, 228, 225],
	'PAPAYAWHIP': [255, 239, 213],
	'LIGHTGOLDENRODYELLOW': [250, 250, 210],
	'CORNSILK': [255, 248, 220],
	'LEMONCHIFFON': [255, 250, 205],
	'LIGHTYELLOW': [255, 255, 224],
	# Gray and black
	'BLACK': [0, 0, 0],
	'DARKGRAY': [64, 64, 64],
	'DIMGRAY': [105, 105, 105],
	'SLATEGRAY': [112, 128, 144],
	'GRAY': [128, 128, 128],
	'LIGHTSLATEGRAY': [119, 136, 153],
	'SILVER': [192, 192, 192],
	'LIGHTGRAY': [211, 211, 211],
	'GAINSBORO': [220, 220, 220],
}

def DecColor(color_name: str) -> list[int, int, int]:
	"""
	Returns a decimal color value.

	:param color_name: Color name
	:return: Decimal color value
	"""
	if color_name in ColorPicker:
		return ColorPicker[color_name]
	else:
		raise ColorException("This color is not in the dictionary")

def HexColor(color_name: str) -> str:
	"""
	Returns a hexadecimal color value.

	:param color_name: Color name
	:return: Hexadecimal color value
	"""
	return '{:02x}{:02x}{:02x}'.format(*DecColor(color_name))

def AnsiColor(color_name: str, color_ground: str) -> str:
	"""
	Returns an ANSI color value.\n
	In AnsiFormat, the following levels are available under the "color/set/..." path:\n
	- foreground
	- background
	- bright foreground
	- bright background
	- underline

	:param color_name: Color name
	:param color_ground: Color level
	:return: ANSI color value
	"""
	return GetAnsiFormat("color/set/{}/{};{};{}".format(color_ground, *DecColor(color_name)))

def Dec2Hex(dec_colors: list[int, int, int]) -> str:
	"""
	Converts a decimal color value to a hexadecimal.

	:param dec_colors: Decimal color value
	:return: Hexadecimal color value
	"""
	return '{:02x}{:02x}{:02x}'.format(*dec_colors)

def Dec2Ansi(dec_colors: list[int, int, int], color_ground: str) -> str:
	"""
	Converts a decimal color value to an ANSI escape code.

	:param dec_colors: Decimal color value
	:param color_ground: Color level (read AnsiColor() function documentation)
	:return: ANSI escape code color value
	"""
	return GetAnsiFormat("color/set/{}/{};{};{}".format(color_ground, *dec_colors))

def Hex2Dec(hex_color: str) -> list[int, int, int]:
	"""
	Converts a hexadecimal color value to a decimal.

	:param hex_color: Hexadecimal color value
	:return: Decimal color value
	"""
	return [
		int(hex_color[:2], base=16),
		int(hex_color[2:4], base=16),
		int(hex_color[4:], base=16)
	]

def Hex2Ansi(hex_color: str, color_ground: str) -> str:
	"""
	Converts a hexadecimal color value to an ANSI escape code.

	:param hex_color: Hexadecimal color value
	:param color_ground: Color level (read AnsiColor() function documentation)
	:return: ANSI escape code color value
	"""
	return GetAnsiFormat("color/set/{}/{};{};{}".format(
		color_ground,
		int(hex_color[:2], base=16),
		int(hex_color[2:4], base=16),
		int(hex_color[4:], base=16)
	))

def Ansi2Dec(ansi_color: str) -> list[int, int, int]:
	"""
	Converts an ANSI escape code color value to a decimal.
	When converting to ANSI escape code, you need to know which level the color is applied
	to, and when from ANSI escape code, you don’t need to, because in other formats the levels
	are defined differently and are not included in the command along with the color.

	:param ansi_color: ANSI escape code color value
	:return: Decimal color value
	"""
	return [
		int(ansi_color.split(';')[2]),
		int(ansi_color.split(';')[3]),
		int(ansi_color.split(';')[4][:-1])
	]

def Ansi2Hex(ansi_color: str) -> str:
	"""
	Converts an ANSI escape code color value to a hexadecimal.
	When converting to ANSI escape code, you need to know which level the color is applied
	to, and when from ANSI escape code, you don’t need to, because in other formats the levels
	are defined differently and are not included in the command along with the color.

	:param ansi_color: ANSI escape code color value
	:return: Hexadecimal color value
	"""
	return '{:02x}{:02x}{:02x}'.format(
		int(ansi_color.split(';')[2]),
		int(ansi_color.split(';')[3]),
		int(ansi_color.split(';')[4][:-1])
	)



"""
A module with a collection of exceptions.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

class ColorException(BaseException):
	"""
	The exception that is thrown when there is no color in any palette.
	"""
	def __init__(self, message):
		self.message = message

	def __str__(self):
		return self.message

class CombinationException(BaseException):
	"""
	An exception that is thrown when composing impossible combinations of boolean flags.
	"""
	def __init__(self, message):
		self.message = message

	def __str__(self):
		return self.message

class ReCreationException(BaseException):
	"""
	The exception that is thrown when an object of the Singleton class is re-created.
	"""
	def __init__(self, message):
		self.message = message

	def __str__(self):
		return self.message



"""
A module with the implementation of a powerful logger.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from mighty_logger.basic.basic_logger import BasicLogger
from mighty_logger.basic.exceptions import ColorException, CombinationException, ReCreationException
from mighty_logger.basic.text_buffer_type import TextBufferType
from mighty_logger.src.color_picker import AnsiColor, HexColor, Dec2Ansi, Dec2Hex
from mighty_logger.src.log_environment import LogEnvironments
from mighty_logger.src.status_variables import StatusMessageType
from mighty_logger.text.icon_set import IconSetType, IconSet1
from mighty_logger.text.text_buffer import BasicTextBuffer, TextBuffer

class Logger(BasicLogger):
	"""
	The Logger class is a class that implements the functionality
	of logging the work of software in different directions.\n
	It has a color output of information, settings for the operation of the log.
	Only one class object can be created!!!\n
	Implements the output of the following information:\n
	1) Record creation time;
	2) Record status;
	3) Recording status message;
	4) Record type;
	5) Write message.
	\nImplements the output of the following types of records:
	see the /docs/DATA.md/"Entry types"
	"""

	def __init__(
			self,
			*,
			program_name: str = "Unknown",
			log_environment: str = LogEnvironments.CONSOLE,
			console_width: int = 60,
			icon_set: IconSetType = IconSet1(),
			time_global_entry: bool = True,
			status_global_entry: bool = True,
			status_message_global_entry: bool = True,
			status_type_global_entry: bool = True,
			message_global_entry: bool = True,
			global_bold_font: bool = False,
			global_italic_font: bool = False,
			global_invert_font: bool = False,
			global_background: bool = False,
	) -> None:
		if not hasattr(self, "_ColorScheme"):
			super().__init__(program_name)
			self._icon_set = icon_set
			self._settings["global_bold_font"] = global_bold_font
			self._settings["global_italic_font"] = global_italic_font
			self._settings["global_invert_font"] = global_invert_font
			self._settings["time_global_entry"] = time_global_entry
			self._settings["status_global_entry"] = status_global_entry
			self._settings["status_message_global_entry"] = status_message_global_entry
			self._settings["status_type_global_entry"] = status_type_global_entry
			self._settings["message_global_entry"] = message_global_entry
			self._ColorScheme: dict = {}
			self._environment = log_environment
			self.global_background = global_background
			self._color_scheme_init()
			if self._environment == LogEnvironments.CONSOLE:
				if TextBuffer._instance is not None:
					self._buffer = TextBuffer._instance
					self.notice(
						message_text="An existing logger was taken into use",
						status_message=StatusMessageType("Note"),
						local_settings={"italic": True}
					)
				else:
					self._buffer = TextBuffer(console_width)
			else:
				if BasicTextBuffer._instance is not None:
					self._buffer = BasicTextBuffer._instance
					self.notice(
						message_text="An existing logger was taken into use",
						status_message=StatusMessageType("Note"),
						local_settings={"italic": True}
					)
				else:
					self._buffer = BasicTextBuffer()
			self._initial_log()
		else:
			raise ReCreationException("Logger class object already created")

	def _color_scheme_init(self) -> None:
		"""
		Sets the colors of the logger.
		"""
		match self._environment:
			case LogEnvironments.CONSOLE:
				self._ColorScheme['INITIAL_COLOR'] = [AnsiColor('GOLD', "foreground"), AnsiColor('INDIGO', "foreground")]
				self._ColorScheme['INITIAL_BACKGROUND'] = ["", AnsiColor('GOLD', "background")]
				# DEBUG colors
				self._ColorScheme['DEBUG_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['DEBUG_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['DEBUG_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_DEBUG'] = [AnsiColor('BURLYWOOD', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['DEBUG_MESSAGE'] = [AnsiColor('TAN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['DEBUG_BACKGROUND'] = ["", AnsiColor('TAN', "background")]
				# DEBUG_PERFORMANCE colors
				self._ColorScheme['DEBUG_PERFORMANCE_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['DEBUG_PERFORMANCE_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['DEBUG_PERFORMANCE_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_DEBUG_PERFORMANCE'] = [AnsiColor('NAVAJOWHITE', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['DEBUG_PERFORMANCE_MESSAGE'] = [AnsiColor('WHEAT', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['DEBUG_PERFORMANCE_BACKGROUND'] = ["", AnsiColor('WHEAT', "background")]
				# PERFORMANCE colors
				self._ColorScheme['PERFORMANCE_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['PERFORMANCE_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['PERFORMANCE_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_PERFORMANCE'] = [AnsiColor('BLANCHEDALMOND', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['PERFORMANCE_MESSAGE'] = [AnsiColor('BISQUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['PERFORMANCE_BACKGROUND'] = ["", AnsiColor('BISQUE', "background")]
				# EVENT colors
				self._ColorScheme['EVENT_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['EVENT_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['EVENT_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_EVENT'] = [AnsiColor('GREENYELLOW', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['EVENT_MESSAGE'] = [AnsiColor('YELLOWGREEN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['EVENT_BACKGROUND'] = ["", AnsiColor('YELLOWGREEN', "background")]
				# AUDIT colors
				self._ColorScheme['AUDIT_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['AUDIT_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['AUDIT_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_AUDIT'] = [AnsiColor('MEDIUMSPRINGGREEN', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['AUDIT_MESSAGE'] = [AnsiColor('SPRINGGREEN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['AUDIT_BACKGROUND'] = ["", AnsiColor('SPRINGGREEN', "background")]
				# METRICS colors
				self._ColorScheme['METRICS_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['METRICS_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['METRICS_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_METRICS'] = [AnsiColor('PALEGREEN', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['METRICS_MESSAGE'] = [AnsiColor('LIGHTGREEN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['METRICS_BACKGROUND'] = ["", AnsiColor('LIGHTGREEN', "background")]
				# USER colors
				self._ColorScheme['USER_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['USER_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['USER_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_USER'] = [AnsiColor('CHARTREUSE', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['USER_MESSAGE'] = [AnsiColor('LAWNGREEN', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['USER_BACKGROUND'] = ["", AnsiColor('LAWNGREEN', "background")]
				# MESSAGE colors
				self._ColorScheme['MESSAGE_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['MESSAGE_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['MESSAGE_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_MESSAGE'] = [AnsiColor('PALETURQUOISE', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['MESSAGE_MESSAGE'] = [AnsiColor('POWDERBLUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['MESSAGE_BACKGROUND'] = ["", AnsiColor('POWDERBLUE', "background")]
				# INFO colors
				self._ColorScheme['INFO_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['INFO_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['INFO_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_INFO'] = [AnsiColor('LIGHTSKYBLUE', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['INFO_MESSAGE'] = [AnsiColor('SKYBLUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['INFO_BACKGROUND'] = ["", AnsiColor('SKYBLUE', "background")]
				# NOTICE colors
				self._ColorScheme['NOTICE_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['NOTICE_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['NOTICE_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_NOTICE'] = [AnsiColor('LIGHTBLUE', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['NOTICE_MESSAGE'] = [AnsiColor('LIGHTSTEELBLUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['NOTICE_BACKGROUND'] = ["", AnsiColor('LIGHTSTEELBLUE', "background")]
				# WARNING colors
				self._ColorScheme['WARNING_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('DARKMAGENTA', "foreground")]
				self._ColorScheme['WARNING_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['WARNING_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_WARNING'] = [AnsiColor('YELLOW', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['WARNING_MESSAGE'] = [AnsiColor('DARKYELLOW', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['WARNING_BACKGROUND'] = ["", AnsiColor('DARKYELLOW', "background")]
				# ERROR colors
				self._ColorScheme['ERROR_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('PLUM', "foreground")]
				self._ColorScheme['ERROR_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('ORANGE', "foreground")]
				self._ColorScheme['ERROR_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('DARKORANGE', "foreground")]
				self._ColorScheme['TYPE_ERROR'] = [AnsiColor('FIREBRICK', "foreground"), AnsiColor('GAINSBORO', "foreground")]
				self._ColorScheme['ERROR_MESSAGE'] = [AnsiColor('DARKRED', "foreground"), AnsiColor('LIGHTGRAY', "foreground")]
				self._ColorScheme['ERROR_BACKGROUND'] = ["", AnsiColor('DARKRED', "background")]
				# CRITICAL colors
				self._ColorScheme['CRITICAL_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('PLUM', "foreground")]
				self._ColorScheme['CRITICAL_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('ORANGE', "foreground")]
				self._ColorScheme['CRITICAL_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('DARKORANGE', "foreground")]
				self._ColorScheme['TYPE_CRITICAL'] = [AnsiColor('FIREBRICK', "foreground"), AnsiColor('DARKSALMON', "foreground")]
				self._ColorScheme['CRITICAL_MESSAGE'] = [AnsiColor('DARKRED', "foreground"), AnsiColor('LIGHTSALMON', "foreground")]
				self._ColorScheme['CRITICAL_BACKGROUND'] = ["", AnsiColor('MAROON', "background")]
				# PROGRESS colors
				self._ColorScheme['PROGRESS_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('PURPLE', "foreground")]
				self._ColorScheme['PROGRESS_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('DARKRED', "foreground")]
				self._ColorScheme['PROGRESS_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('MAROON', "foreground")]
				self._ColorScheme['TYPE_PROGRESS'] = [AnsiColor('LIGHTSKYBLUE', "foreground"), AnsiColor('NAVY', "foreground")]
				self._ColorScheme['PROGRESS_MESSAGE'] = [AnsiColor('SKYBLUE', "foreground"), AnsiColor('MIDNIGHTBLUE', "foreground")]
				self._ColorScheme['PROGRESS_BACKGROUND'] = ["", AnsiColor('SKYBLUE', "background")]
				# SUCCESS colors
				self._ColorScheme['SUCCESS_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('LAVENDERBLUSH', "foreground")]
				self._ColorScheme['SUCCESS_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('CHARTREUSE', "foreground")]
				self._ColorScheme['SUCCESS_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('LAWNGREEN', "foreground")]
				self._ColorScheme['TYPE_SUCCESS'] = [AnsiColor('GREEN', "foreground"), AnsiColor('PALEGREEN', "foreground")]
				self._ColorScheme['SUCCESS_MESSAGE'] = [AnsiColor('DARKGREEN', "foreground"), AnsiColor('LIGHTGREEN', "foreground")]
				self._ColorScheme['SUCCESS_BACKGROUND'] = ["", AnsiColor('DARKGREEN', "background")]
				# FAIL colors
				self._ColorScheme['FAIL_TIME'] = [AnsiColor('ORCHID', "foreground"), AnsiColor('LAVENDERBLUSH', "foreground")]
				self._ColorScheme['FAIL_STATUS'] = [AnsiColor('ORANGE', "foreground"), AnsiColor('ORANGE', "foreground")]
				self._ColorScheme['FAIL_STATUS_MESSAGE'] = [AnsiColor('DARKORANGE', "foreground"), AnsiColor('DARKORANGE', "foreground")]
				self._ColorScheme['TYPE_FAIL'] = [AnsiColor('FIREBRICK', "foreground"), AnsiColor('YELLOW', "foreground")]
				self._ColorScheme['FAIL_MESSAGE'] = [AnsiColor('DARKRED', "foreground"), AnsiColor('DARKYELLOW', "foreground")]
				self._ColorScheme['FAIL_BACKGROUND'] = ["", AnsiColor('DARKRED', "background")]
			case LogEnvironments.HTML:
				self._ColorScheme['INITIAL_COLOR'] = [HexColor('GOLD'), HexColor('INDIGO')]
				self._ColorScheme['INITIAL_BACKGROUND'] = ["", HexColor('GOLD')]
				# DEBUG colors
				self._ColorScheme['DEBUG_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['DEBUG_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['DEBUG_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_DEBUG'] = [HexColor('BURLYWOOD'), HexColor('NAVY')]
				self._ColorScheme['DEBUG_MESSAGE'] = [HexColor('TAN'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['DEBUG_BACKGROUND'] = ["", HexColor('TAN')]
				# DEBUG_PERFORMANCE colors
				self._ColorScheme['DEBUG_PERFORMANCE_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['DEBUG_PERFORMANCE_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['DEBUG_PERFORMANCE_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_DEBUG_PERFORMANCE'] = [HexColor('NAVAJOWHITE'), HexColor('NAVY')]
				self._ColorScheme['DEBUG_PERFORMANCE_MESSAGE'] = [HexColor('WHEAT'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['DEBUG_PERFORMANCE_BACKGROUND'] = ["", HexColor('WHEAT')]
				# PERFORMANCE colors
				self._ColorScheme['PERFORMANCE_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['PERFORMANCE_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['PERFORMANCE_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_PERFORMANCE'] = [HexColor('BLANCHEDALMOND'), HexColor('NAVY')]
				self._ColorScheme['PERFORMANCE_MESSAGE'] = [HexColor('BISQUE'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['PERFORMANCE_BACKGROUND'] = ["", HexColor('BISQUE')]
				# EVENT colors
				self._ColorScheme['EVENT_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['EVENT_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['EVENT_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_EVENT'] = [HexColor('GREENYELLOW'), HexColor('NAVY')]
				self._ColorScheme['EVENT_MESSAGE'] = [HexColor('YELLOWGREEN'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['EVENT_BACKGROUND'] = ["", HexColor('YELLOWGREEN')]
				# AUDIT colors
				self._ColorScheme['AUDIT_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['AUDIT_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['AUDIT_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_AUDIT'] = [HexColor('MEDIUMSPRINGGREEN'), HexColor('NAVY')]
				self._ColorScheme['AUDIT_MESSAGE'] = [HexColor('SPRINGGREEN'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['AUDIT_BACKGROUND'] = ["", HexColor('SPRINGGREEN')]
				# METRICS colors
				self._ColorScheme['METRICS_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['METRICS_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['METRICS_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_METRICS'] = [HexColor('PALEGREEN'), HexColor('NAVY')]
				self._ColorScheme['METRICS_MESSAGE'] = [HexColor('LIGHTGREEN'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['METRICS_BACKGROUND'] = ["", HexColor('LIGHTGREEN')]
				# USER colors
				self._ColorScheme['USER_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['USER_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['USER_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_USER'] = [HexColor('CHARTREUSE'), HexColor('NAVY')]
				self._ColorScheme['USER_MESSAGE'] = [HexColor('LAWNGREEN'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['USER_BACKGROUND'] = ["", HexColor('LAWNGREEN')]
				# MESSAGE colors
				self._ColorScheme['MESSAGE_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['MESSAGE_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['MESSAGE_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_MESSAGE'] = [HexColor('PALETURQUOISE'), HexColor('NAVY')]
				self._ColorScheme['MESSAGE_MESSAGE'] = [HexColor('POWDERBLUE'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['MESSAGE_BACKGROUND'] = ["", HexColor('POWDERBLUE')]
				# INFO colors
				self._ColorScheme['INFO_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['INFO_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['INFO_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_INFO'] = [HexColor('LIGHTSKYBLUE'), HexColor('NAVY')]
				self._ColorScheme['INFO_MESSAGE'] = [HexColor('SKYBLUE'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['INFO_BACKGROUND'] = ["", HexColor('SKYBLUE')]
				# NOTICE colors
				self._ColorScheme['NOTICE_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['NOTICE_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['NOTICE_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_NOTICE'] = [HexColor('LIGHTBLUE'), HexColor('NAVY')]
				self._ColorScheme['NOTICE_MESSAGE'] = [HexColor('LIGHTSTEELBLUE'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['NOTICE_BACKGROUND'] = ["", HexColor('LIGHTSTEELBLUE')]
				# WARNING colors
				self._ColorScheme['WARNING_TIME'] = [HexColor('ORCHID'), HexColor('DARKMAGENTA')]
				self._ColorScheme['WARNING_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['WARNING_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_WARNING'] = [HexColor('YELLOW'), HexColor('NAVY')]
				self._ColorScheme['WARNING_MESSAGE'] = [HexColor('DARKYELLOW'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['WARNING_BACKGROUND'] = ["", HexColor('DARKYELLOW')]
				# ERROR colors
				self._ColorScheme['ERROR_TIME'] = [HexColor('ORCHID'), HexColor('PLUM')]
				self._ColorScheme['ERROR_STATUS'] = [HexColor('ORANGE'), HexColor('ORANGE')]
				self._ColorScheme['ERROR_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('DARKORANGE')]
				self._ColorScheme['TYPE_ERROR'] = [HexColor('FIREBRICK'), HexColor('GAINSBORO')]
				self._ColorScheme['ERROR_MESSAGE'] = [HexColor('DARKRED'), HexColor('LIGHTGRAY')]
				self._ColorScheme['ERROR_BACKGROUND'] = ["", HexColor('DARKRED')]
				# CRITICAL colors
				self._ColorScheme['CRITICAL_TIME'] = [HexColor('ORCHID'), HexColor('PLUM')]
				self._ColorScheme['CRITICAL_STATUS'] = [HexColor('ORANGE'), HexColor('ORANGE')]
				self._ColorScheme['CRITICAL_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('DARKORANGE')]
				self._ColorScheme['TYPE_CRITICAL'] = [HexColor('FIREBRICK'), HexColor('DARKSALMON')]
				self._ColorScheme['CRITICAL_MESSAGE'] = [HexColor('DARKRED'), HexColor('LIGHTSALMON')]
				self._ColorScheme['CRITICAL_BACKGROUND'] = ["", HexColor('MAROON')]
				# PROGRESS colors
				self._ColorScheme['PROGRESS_TIME'] = [HexColor('ORCHID'), HexColor('PURPLE')]
				self._ColorScheme['PROGRESS_STATUS'] = [HexColor('ORANGE'), HexColor('DARKRED')]
				self._ColorScheme['PROGRESS_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('MAROON')]
				self._ColorScheme['TYPE_PROGRESS'] = [HexColor('LIGHTSKYBLUE'), HexColor('NAVY')]
				self._ColorScheme['PROGRESS_MESSAGE'] = [HexColor('SKYBLUE'), HexColor('MIDNIGHTBLUE')]
				self._ColorScheme['PROGRESS_BACKGROUND'] = ["", HexColor('SKYBLUE')]
				# SUCCESS colors
				self._ColorScheme['SUCCESS_TIME'] = [HexColor('ORCHID'), HexColor('LAVENDERBLUSH')]
				self._ColorScheme['SUCCESS_STATUS'] = [HexColor('ORANGE'), HexColor('CHARTREUSE')]
				self._ColorScheme['SUCCESS_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('LAWNGREEN')]
				self._ColorScheme['TYPE_SUCCESS'] = [HexColor('GREEN'), HexColor('PALEGREEN')]
				self._ColorScheme['SUCCESS_MESSAGE'] = [HexColor('DARKGREEN'), HexColor('LIGHTGREEN')]
				self._ColorScheme['SUCCESS_BACKGROUND'] = ["", HexColor('DARKGREEN')]
				# FAIL colors
				self._ColorScheme['FAIL_TIME'] = [HexColor('ORCHID'), HexColor('LAVENDERBLUSH')]
				self._ColorScheme['FAIL_STATUS'] = [HexColor('ORANGE'), HexColor('ORANGE')]
				self._ColorScheme['FAIL_STATUS_MESSAGE'] = [HexColor('DARKORANGE'), HexColor('DARKORANGE')]
				self._ColorScheme['TYPE_FAIL'] = [HexColor('FIREBRICK'), HexColor('YELLOW')]
				self._ColorScheme['FAIL_MESSAGE'] = [HexColor('DARKRED'), HexColor('DARKYELLOW')]
				self._ColorScheme['FAIL_BACKGROUND'] = ["", HexColor('DARKRED')]

	def _initial_log(self) -> None:
		"""
		Displays initialized information.
		"""
		if self._environment == LogEnvironments.HTML:
			self._buffer << "<body style='background-color: #000000; color: #ffffff;'>"
		self._buffer << self._initialized_data(
			[
				self._ColorScheme['INITIAL_COLOR'][self.global_background],
				self._ColorScheme['INITIAL_BACKGROUND'][self.global_background]
			], self._environment
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	# def set_animation(self):
	# 	pass

	def set_icons(self, icon_set: IconSetType):
		"""
		Changes the current icon set used by the Logger.

		:param icon_set: Icon set to use
		"""
		self._icon_set = icon_set

	def set_color(self, *, logger_color_name: str, color_value: list[int, int, int], foreground: bool = True, background: bool = False) -> None:
		"""
		A method that sets the ANSI escape code color in the color table of the logger.
		May throw a ColorException if the given color is not in the table.
		The logger color table stores the following keys: see /docs/DATA.md/"Logger Color Scheme".\n
		Boolean flags: If foreground is set to True, then the color of the foreground text will change
		with/without a background (it all depends on the background flag). If in this case background
		is set to False (the standard combination of arguments) - then the color of the specifically
		front text that is displayed without a background changes, otherwise it changes the color
		of the specifically front text that is displayed with a background. If the foreground is set
		to False with background set to True, the background itself will change. The last combination,
		when both arguments are False, is an impossible combination that throws a CombinationException.

		:param logger_color_name: Color name in logger color table
		:param color_value: Color value in RGB
		:param foreground: Change foreground text color with/without background?
		:param background: Change background color?
		"""
		if logger_color_name in self._ColorScheme:
			if background and not foreground:
				self._ColorScheme[logger_color_name][1] = Dec2Ansi(color_value, "background") if self._environment == LogEnvironments.CONSOLE else Dec2Hex(color_value)
			elif background and foreground:
				self._ColorScheme[logger_color_name][1] = Dec2Ansi(color_value, "foreground") if self._environment == LogEnvironments.CONSOLE else Dec2Hex(color_value)
			elif not background and foreground:
				self._ColorScheme[logger_color_name][0] = Dec2Ansi(color_value, "foreground") if self._environment == LogEnvironments.CONSOLE else Dec2Hex(color_value)
			else:
				raise CombinationException("False-False combination of foreground-background flags not possible")
		else:
			raise ColorException("This color is not in the dictionary")

	def buffer(self) -> TextBufferType:
		"""
		The Text Buffer object is created in the class constructor and this
		method is used to access it. It returns a buffer.

		:return: a text buffer object
		"""
		return self._buffer

	#todo v0.7.1 сделать конвертер из Console в HTML и наоборот

	def debug(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		Debugging information logging:
		Can be used to log entry any information while debugging an application.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['DEBUG_TIME'][background],
				self._ColorScheme['DEBUG_STATUS'][background],
				self._ColorScheme['DEBUG_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_DEBUG'][background],
				self._ColorScheme['DEBUG_MESSAGE'][background],
				self._ColorScheme['DEBUG_BACKGROUND'][background],
			], [""], self._icon_set.debug, status_message.current_status_message, "%DEBUG", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def debug_performance(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		Performance debugging information logging:
		Can be used to log entry the execution time of operations or other
		performance information while the application is being debugged.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['DEBUG_PERFORMANCE_TIME'][background],
				self._ColorScheme['DEBUG_PERFORMANCE_STATUS'][background],
				self._ColorScheme['DEBUG_PERFORMANCE_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_DEBUG_PERFORMANCE'][background],
				self._ColorScheme['DEBUG_PERFORMANCE_MESSAGE'][background],
				self._ColorScheme['DEBUG_PERFORMANCE_BACKGROUND'][background],
			], [""], self._icon_set.debug_performance, status_message.current_status_message, "%DEBUG PERFORMANCE", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def performance(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		Performance information logging:
		Can be used to log entry the execution time of operations or
		other application performance information.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['PERFORMANCE_TIME'][background],
				self._ColorScheme['PERFORMANCE_STATUS'][background],
				self._ColorScheme['PERFORMANCE_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_PERFORMANCE'][background],
				self._ColorScheme['PERFORMANCE_MESSAGE'][background],
				self._ColorScheme['PERFORMANCE_BACKGROUND'][background],
			], [""], self._icon_set.performance, status_message.current_status_message, "%PERFORMANCE", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def event(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		Event information logging:
		Can be used to log entry specific events in the application,
		such as button presses or mouse cursor movements.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['EVENT_TIME'][background],
				self._ColorScheme['EVENT_STATUS'][background],
				self._ColorScheme['EVENT_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_EVENT'][background],
				self._ColorScheme['EVENT_MESSAGE'][background],
				self._ColorScheme['EVENT_BACKGROUND'][background],
			], [""], self._icon_set.event, status_message.current_status_message, "~EVENT", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def audit(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		Audit information logging:
		Can be used to log entry changes in the system, such as creating or
		deleting users, as well as changes in security settings.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['AUDIT_TIME'][background],
				self._ColorScheme['AUDIT_STATUS'][background],
				self._ColorScheme['AUDIT_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_AUDIT'][background],
				self._ColorScheme['AUDIT_MESSAGE'][background],
				self._ColorScheme['AUDIT_BACKGROUND'][background],
			], [""], self._icon_set.audit, status_message.current_status_message, "~AUDIT", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def metrics(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		Metrics information logging:
		Can be used to log entry metrics to track application performance and identify issues.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['METRICS_TIME'][background],
				self._ColorScheme['METRICS_STATUS'][background],
				self._ColorScheme['METRICS_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_METRICS'][background],
				self._ColorScheme['METRICS_MESSAGE'][background],
				self._ColorScheme['METRICS_BACKGROUND'][background],
			], [""], self._icon_set.metrics, status_message.current_status_message, "~METRICS", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def user(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		User information logging:
		Can be used to log entry custom logs to store additional information
		that may be useful for diagnosing problems.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['USER_TIME'][background],
				self._ColorScheme['USER_STATUS'][background],
				self._ColorScheme['USER_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_USER'][background],
				self._ColorScheme['USER_MESSAGE'][background],
				self._ColorScheme['USER_BACKGROUND'][background],
			], [""], self._icon_set.user, status_message.current_status_message, "~USER", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def message(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		Message information logging:
		Can be used for the usual output of ordinary messages about the program's operation.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['MESSAGE_TIME'][background],
				self._ColorScheme['MESSAGE_STATUS'][background],
				self._ColorScheme['MESSAGE_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_MESSAGE'][background],
				self._ColorScheme['MESSAGE_MESSAGE'][background],
				self._ColorScheme['MESSAGE_BACKGROUND'][background],
			], [""], self._icon_set.message, status_message.current_status_message, "@MESSAGE", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def info(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		Default information logging:
		Can be used to log entry messages with specific content about the operation of the program.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['INFO_TIME'][background],
				self._ColorScheme['INFO_STATUS'][background],
				self._ColorScheme['INFO_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_INFO'][background],
				self._ColorScheme['INFO_MESSAGE'][background],
				self._ColorScheme['INFO_BACKGROUND'][background],
			], [""], self._icon_set.info, status_message.current_status_message, "@INFO", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def notice(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = None, local_settings: dict = None) -> None:
		"""
		Notice information logging:
		Can be used to flag important events that might be missed with a normal logging level.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		background = local_background if local_background is not None else self.global_background
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['NOTICE_TIME'][background],
				self._ColorScheme['NOTICE_STATUS'][background],
				self._ColorScheme['NOTICE_STATUS_MESSAGE'][background],
				self._ColorScheme['TYPE_NOTICE'][background],
				self._ColorScheme['NOTICE_MESSAGE'][background],
				self._ColorScheme['NOTICE_BACKGROUND'][background],
			], [""], self._icon_set.notice, status_message.current_status_message, "@NOTICE", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def warning(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = True, local_settings: dict = None) -> None:
		"""
		Warning information logging:
		Can be used to log entry warnings that the program may work with unpredictable results.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['WARNING_TIME'][local_background],
				self._ColorScheme['WARNING_STATUS'][local_background],
				self._ColorScheme['WARNING_STATUS_MESSAGE'][local_background],
				self._ColorScheme['TYPE_WARNING'][local_background],
				self._ColorScheme['WARNING_MESSAGE'][local_background],
				self._ColorScheme['WARNING_BACKGROUND'][local_background],
			], [""], self._icon_set.warning, status_message.current_status_message, "!WARNING", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def error(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = True, local_settings: dict = None) -> None:
		"""
		Error information logging:
		Used to log entry errors and crashes in the program.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['ERROR_TIME'][local_background],
				self._ColorScheme['ERROR_STATUS'][local_background],
				self._ColorScheme['ERROR_STATUS_MESSAGE'][local_background],
				self._ColorScheme['TYPE_ERROR'][local_background],
				self._ColorScheme['ERROR_MESSAGE'][local_background],
				self._ColorScheme['ERROR_BACKGROUND'][local_background],
			], [""], self._icon_set.error, status_message.current_status_message, "!!ERROR", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def critical(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = True, local_settings: dict = None) -> None:
		"""
		Critical error information logging:
		Used to log entry for critical and unpredictable program failures.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		if not 'bold' in local_settings:
			local_settings["bold"] = True
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['CRITICAL_TIME'][local_background],
				self._ColorScheme['CRITICAL_STATUS'][local_background],
				self._ColorScheme['CRITICAL_STATUS_MESSAGE'][local_background],
				self._ColorScheme['TYPE_CRITICAL'][local_background],
				self._ColorScheme['CRITICAL_MESSAGE'][local_background],
				self._ColorScheme['CRITICAL_BACKGROUND'][local_background],
			], [""], self._icon_set.critical, status_message.current_status_message, "!!!@CRITICAL", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def start_process(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = True, local_settings: dict = None) -> None:
		"""
		Stub.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['PROGRESS_TIME'][local_background],
				self._ColorScheme['PROGRESS_STATUS'][local_background],
				self._ColorScheme['PROGRESS_STATUS_MESSAGE'][local_background],
				self._ColorScheme['TYPE_PROGRESS'][local_background],
				self._ColorScheme['PROGRESS_MESSAGE'][local_background],
				self._ColorScheme['PROGRESS_BACKGROUND'][local_background],
			], [""], self._icon_set.process, status_message.current_status_message, "&PROGRESS [*******.............] - 37%", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()
		# pass
		# Must run on a thread

	def stop_process(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = True, local_settings: dict = None) -> None:
		"""
		Stub.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		pass
		# Make transition to SUCCESS or FAIL

	def success(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = True, local_settings: dict = None) -> None:
		"""
		Success information logging:
		Used to log entry a message about the success of the process.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		if not 'italic' in local_settings:
			local_settings["italic"] = True
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['SUCCESS_TIME'][local_background],
				self._ColorScheme['SUCCESS_STATUS'][local_background],
				self._ColorScheme['SUCCESS_STATUS_MESSAGE'][local_background],
				self._ColorScheme['TYPE_SUCCESS'][local_background],
				self._ColorScheme['SUCCESS_MESSAGE'][local_background],
				self._ColorScheme['SUCCESS_BACKGROUND'][local_background],
			], [""], self._icon_set.success, status_message.current_status_message, "&SUCCESS", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()

	def fail(self, *, status_message: StatusMessageType = StatusMessageType("..."), message_text: str = "...", local_background: bool = True, local_settings: dict = None) -> None:
		"""
		Fail information logging:
		Used to log entry a message about the failed execution of the process.

		:param status_message: Log entry status message
		:param message_text: Log entry message
		:param local_background: Display entry with background?
		:param local_settings: Dictionary of local recording settings
		"""
		if local_settings is None:
			local_settings = {}
		if not 'italic' in local_settings:
			local_settings["italic"] = True
		self._buffer << self._assemble_entry(
			[
				self._ColorScheme['FAIL_TIME'][local_background],
				self._ColorScheme['FAIL_STATUS'][local_background],
				self._ColorScheme['FAIL_STATUS_MESSAGE'][local_background],
				self._ColorScheme['TYPE_FAIL'][local_background],
				self._ColorScheme['FAIL_MESSAGE'][local_background],
				self._ColorScheme['FAIL_BACKGROUND'][local_background],
			], [""], self._icon_set.fail, status_message.current_status_message, "&FAIL", message_text, self._environment, local_settings
		)
		if self._environment == LogEnvironments.CONSOLE:
			self._buffer.update_console()



"""
A module with an implementation of the type for text buffers.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from abc import ABC, abstractmethod

class TextBufferType(ABC):
	def __init__(self) -> None:
		self._text_buffer: list[str] = []

	def __lshift__(self, entry: str) -> None:
		"""
		Used to add a string to the end of the buffer.

		:param entry: The line to be added
		"""
		self.append(f"{entry}")

	def __rshift__(self, entry: str) -> None:
		"""
		Used to save a buffer to the file.

		:param entry: The name of the file where you want to save the buffer
		"""
		self.save(entry)

	def get_data(self) -> list:
		"""
		Returns a list of strings from a text buffer.

		:return: a list of text buffer lines
		"""
		return self._text_buffer

	@abstractmethod
	def append(self, message: str) -> None:
		"""
		Adds a string to the end of the text buffer.

		:param message: The string to be added to the buffer
		"""
		raise NotImplementedError("Method append() is not implemented in the base class.")

	@abstractmethod
	def insert(self, number_string: int, message: str) -> None:
		"""
		Adds a string to the middle of the text buffer at the specified position.

		:param number_string: Position (number) of the line to which you need to add a string
		:param message: The string to be placed on the position
		"""
		raise NotImplementedError("Method insert() is not implemented in the base class.")

	@abstractmethod
	def replace(self, number_string: int, message: str) -> None:
		"""
		Replaces a specific string in a text buffer. If there is no such string, the method
		fills the list with empty strings up to the required position and *adds* the string.

		:param number_string: Position (number) of the string to be replaced (added)
		:param message: A string that will replace the previous one by position
		"""
		raise NotImplementedError("Method replace() is not implemented in the base class.")

	@abstractmethod
	def save(self, name_file: str = "buffer", clean: bool = True) -> None:
		"""
		Saves the text of the buffer to a file.

		:param name_file: The name of the file where the buffer will be saved
		:param clean: Saving should be done in Plain text?
		"""
		raise NotImplementedError("Method save() is not implemented in the base class.")

	@abstractmethod
	def update_console(self) -> None:
		"""
		Refreshes the console, erasing output text and outputting an updated buffer.
		"""
		raise NotImplementedError("Method update_console() is not implemented in the base class.")

	# todo v0.7.0 abstractmethod remove(), clear(), -open_save() or load()-



"""
Module with implementation of ANSI escape codes.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

AnsiFormat = {
	# Source: https://en.wikipedia.org/wiki/ANSI_escape_code
	'reset': {
		'on': '\033[0m',
	},

	'bold': {
		'on': '\033[1m',
		'off (doubly underlined)': '\033[21m',
	},

	'faint': {
		'on': '\033[2m',
		'off': '\033[22m',
	},

	'italic': {
		'on': '\033[3m',
		'fraktur': '\033[20m',
		'off': '\033[23m',
	},

	'underline': {
		'on': '\033[4m',
		'off': '\033[24m',
	},

	'blink': {  # not work
		'slow': '\033[5m',
		'rapid': '\033[6m',
		'off': '\033[25m',
	},

	'proportional spacing': {  # not work
		'on': '\033[26m',
		'off': '\033[50m',
	},

	'invert': {
		'on': '\033[7m',
		'off': '\033[27m',
	},

	'hide': {
		'on': '\033[8m',
		'off': '\033[28m',
	},

	'strike': {
		'on': '\033[9m',
		'off': '\033[29m',
	},

	'over line': {
		'on': '\033[53m',
		'off': '\033[55m',
	},

	'framed': {
		'on': '\033[51m',
		'encircled': '\033[52m',
		'off': '\033[54m',
	},

	'font': {
		'primary': '\033[10m',
		'1st alternative': '\033[11m',
		'2nd alternative': '\033[12m',
		'3rd alternative': '\033[13m',
		'4th alternative': '\033[14m',
		'5th alternative': '\033[15m',
		'6th alternative': '\033[16m',
		'7th alternative': '\033[17m',
		'8th alternative': '\033[18m',
		'9th alternative': '\033[19m',
	},

	'color': {
		'foreground': {
			'black': '\033[30m',
			'red': '\033[31m',
			'green': '\033[32m',
			'yellow': '\033[33m',
			'blue': '\033[34m',
			'magenta': '\033[35m',
			'cyan': '\033[36m',
			'white': '\033[37m',
		},
		'background': {
			'black': '\033[40m',
			'red': '\033[41m',
			'green': '\033[42m',
			'yellow': '\033[43m',
			'blue': '\033[44m',
			'magenta': '\033[45m',
			'cyan': '\033[46m',
			'white': '\033[47m',
		},
		'bright foreground': {
			'black': '\033[90m',
			'red': '\033[91m',
			'green': '\033[92m',
			'yellow': '\033[93m',
			'blue': '\033[94m',
			'magenta': '\033[95m',
			'cyan': '\033[96m',
			'white': '\033[97m',
		},
		'bright background': {
			'black': '\033[100m',
			'red': '\033[101m',
			'green': '\033[102m',
			'yellow': '\033[103m',
			'blue': '\033[104m',
			'magenta': '\033[105m',
			'cyan': '\033[106m',
			'white': '\033[107m',
		},
		'set': {
			'foreground': '\033[38;2;$m',
			'background': '\033[48;2;$m',
			'bright foreground': '\033[98;2;$m',
			'bright background': '\033[108;2;$m',
			'underline': '\033[58;2;$m',
		},
		'default': {
			'foreground': '\033[39m',
			'background': '\033[49m',
			'bright foreground': '\033[99m',
			'bright background': '\033[109m',
			'underline': '\033[59m',
		},
	},
}

def _RecursiveGetAnsiFormat(ansi_address: str, ansi: dict) -> str:
	"""
	Recursively extracts a string with an ANSI escape code from a heavily nested dictionary.

	:param ansi_address: Path to ANSI escape code value
	:param ansi: External/nested dictionary
	:return: value - ANSI escape code
	"""
	split_address = ansi_address.split("/")
	# print(split_address)
	if type(ansi[split_address[0]]) == dict:
		return _RecursiveGetAnsiFormat("/".join(split_address[1:]), ansi[split_address[0]])
	else:
		if len(split_address) == 2:
			return ansi[split_address[0]].replace('$', split_address[1])
		else:
			return ansi[split_address[0]]

def GetAnsiFormat(ansi_address: str) -> str:
	"""
	Returns the ANSI escape code value.\n
	The following values are possible: see the list inREADME.md/Data/"Tree of ANSI escape code"\n
	An example of getting an ANSI escape code:\n
	print(f"{GetAnsiFormat('italic/fraktur')}Test string")\n
	print(f"{GetAnsiFormat('blink/slow')}Test string")\n
	print(f"{GetAnsiFormat('invert/off')}Test string")\n
	print(f"{GetAnsiFormat('font/3th alternative')}Test string")\n
	print(f"{GetAnsiFormat('color/foreground/green')}Test string")\n
	print(f"{GetAnsiFormat('color/set/background/255;255;255')}Test string")\n
	print(f"{GetAnsiFormat('reset/on')}Test string")\n

	:param ansi_address: Path to ANSI escape code value
	:return: ANSI escape code
	"""
	return _RecursiveGetAnsiFormat(ansi_address, AnsiFormat)



"""
Module with implementation of text buffers.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import sys, re

from mighty_logger.basic.exceptions import ReCreationException
from mighty_logger.basic.patterns import Singleton
from mighty_logger.basic.text_buffer_type import TextBufferType

class BasicTextBuffer(Singleton, TextBufferType):
	"""
	A class with a basic implementation of a simple text buffer. It is intended
	to be used in conjunction with HTML, but this is optional.
	"""

	def __init__(self) -> None:
		if not hasattr(self, "_text_buffer"):
			super().__init__()
		else:
			raise ReCreationException("BasicTextBuffer class object already created")

	def append(self, message: str) -> None:
		self._text_buffer.append(f"{message}")

	def insert(self, number_string: int, message: str) -> None:
		if number_string < len(self._text_buffer):
			self._text_buffer.insert(number_string, f"{message}")
		else:
			self._text_buffer.extend([""] * (number_string - len(self._text_buffer)))
			self.append(message)

	def replace(self, number_string: int, message: str) -> None:
		if number_string < len(self._text_buffer):
			self._text_buffer[number_string] = f"{message}"
		else:
			self._text_buffer.extend([""] * (number_string - len(self._text_buffer)))
			self.append(message)

	def save(self, name_file: str = "buffer", clean: bool = True) -> None:
		with open(name_file, "w", encoding="utf-8") as text_buffer_file:
			if clean:
				text_buffer_file.write(self._text_buffer[0] + '\n' + '\n<br>'.join(self._text_buffer[1:]))
			else:
				text_buffer_file.write(self._text_buffer[0] + '\n' + '\n'.join(self._text_buffer[1:]))

	def update_console(self) -> None:
		super().update_console()

class TextBuffer(Singleton, TextBufferType):
	"""
	A class with an advanced implementation of the console text buffer. It is not necessary to use it
	only in the console, but almost all methods are reimplemented for more complex algorithms, taking
	into account the width of the console (number of characters per line) and use ANSI escape codes
	that are only found in the console.
	"""

	def __init__(self, console_width: int = 60) -> None:
		if not hasattr(self, "_text_buffer"):
			super().__init__()
			self._cursor_string: int = 0
			self._buffer_size: int = 0
			self.width = console_width
		else:
			raise ReCreationException("TextBuffer class object already created")

	def append(self, message: str) -> None:
		excess_console_string = len(re.sub(r"\033\[.*?m", "", message)) // self.width
		self._buffer_size += 1 + excess_console_string
		self._text_buffer.append(f"{message}")

	def insert(self, number_string: int, message: str) -> None:
		if number_string > self._cursor_string:
			count_empty_strings = (number_string - len(self._text_buffer))
			self._text_buffer.extend([""] * count_empty_strings)
			self._buffer_size += count_empty_strings
			self.append(message)
		else:
			excess_console_string = len(re.sub(r"\033\[.*?m", "", message)) // self.width
			self._buffer_size += 1 + excess_console_string
			self._text_buffer.insert(number_string, f"{message}")

	def replace(self, number_string: int, message: str) -> None:
		if number_string > self._cursor_string:
			count_empty_strings = (number_string - len(self._text_buffer))
			self._text_buffer.extend([""] * count_empty_strings)
			self._buffer_size += count_empty_strings
			self.append(message)
		else:
			old_excess_console_strings = len(re.sub(r"\033\[.*?m", "", self._text_buffer[number_string])) // self.width
			new_excess_console_strings = len(re.sub(r"\033\[.*?m", "", message)) // self.width
			self._buffer_size += new_excess_console_strings - old_excess_console_strings
			self._text_buffer[number_string] = f"{message}"

	def save(self, name_file: str = "buffer", clean: bool = True) -> None:
		with open(name_file, "w", encoding="utf-8") as text_buffer_file:
			if clean:
				for item in self._text_buffer:
					text_buffer_file.write("{}\n".format(re.sub(r"\033\[.*?m", "", item)))
			else:
				text_buffer_file.write('\n'.join(self._text_buffer))

	def update_console(self) -> None:
		# todo Translate to thread in a future update
		if self._cursor_string == 0:
			sys.stdout.write(f'\r\033[K')
		else:
			sys.stdout.write(f'\033[{self._cursor_string}A\r\033[J')
		sys.stdout.write('\n'.join(self._text_buffer))
		sys.stdout.flush()  # Clearing the output buffer so that the changes are displayed immediately
		self._cursor_string = self._buffer_size - 1



from mighty_logger import Logger
from mighty_logger.src import StatusMessagePatterns

if __name__ == "__main__":
	logger = Logger(program_name="Test", console_width=115)
	logger.debug(message_text="Hello world!")
	logger.debug(message_text="Hello world!", local_settings={"status_message_local_entry": False})
	logger.debug(status_message=StatusMessagePatterns.activated(), message_text="Hello world!")
	logger.debug(status_message=StatusMessagePatterns.custom("Hooray"), message_text="Hello world!")
	logger.debug_performance(message_text="Hello world!")
	logger.performance(message_text="Hello world!")
	logger.event(message_text="Hello world!")
	logger.audit(message_text="Hello world!")
	logger.metrics(message_text="Hello world!")
	logger.user(message_text="Hello world!")
	logger.message(message_text="Hello world!")
	logger.info(message_text="Hello world!")
	logger.notice(message_text="Hello world!")
	logger.warning(message_text="Hello world!")
	logger.error(message_text="Hello world!")
	logger.critical(message_text="Hello world!")
	logger.start_process(message_text="Hello world!")
	logger.success(message_text="Hello world!")
	logger.fail(message_text="Hello world!")
	logger.buffer().save("log.txt")



# ##########################   Qt_Сolored-logger   ########################### #
# ---------------------------------------------------------------------------- #
#                                                                              #
# Copyright © 2023 Kalynovsky Valentin. All rights reserved.                   #
#                                                                              #
# Licensed under the Apache License, Version 2.0 (the "License");              #
# you may not use this file except in compliance with the License.             #
# You may obtain a copy of the License at                                      #
#                                                                              #
#     http://www.apache.org/licenses/LICENSE-2.0                               #
#                                                                              #
# Unless required by applicable law or agreed to in writing, software          #
# distributed under the License is distributed on an "AS IS" BASIS,            #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.     #
# See the License for the specific language governing permissions and          #
# limitations under the License.                                               #
#                                                                              #
# ---------------------------------------------------------------------------- #
# ############################################################################ #

# To compile and publish the library, you need to enter the following commands:
# python setup.py sdist
# twine upload dist/*

from setuptools import setup

with open("README.md", "r", encoding='utf-8') as readme_file:
    readme = readme_file.read()

setup(
    name="mighty_logger",
    version="0.5.1",

    author="Kalynovsky 'Nakamura Akira' Valentin",
    author_email="nakama3942@gmail.com",

    description="Powerful functional logger with support for qt programming",
    long_description=readme,
    long_description_content_type="text/markdown",

    url="https://github.com/Nakama3942/mighty_logger",

    license="Apache License, Version 2.0, see LICENSE file",

    packages=[
        'mighty_logger',
        'mighty_logger.text',
        'mighty_logger.src',
        'mighty_logger.basic'
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.11",
        "Topic :: System :: Logging",
    ],
    project_urls={
        'Releases': 'https://github.com/Nakama3942/mighty_logger/releases',
    },
    python_requires='>=3.11',
)



"""
Module with implementation of entry status messages.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

class StatusMessageType:
	"""
	Wrapper class for status message.
	"""
	def __init__(self, message: str):
		self.__current_status_message = message

	@property
	def current_status_message(self) -> str:
		return self.__current_status_message

class StatusMessagePatterns:
	"""
	A class that emits the desired status message templates.
	"""
	@staticmethod
	def completed() -> StatusMessageType:
		return StatusMessageType("Completed")

	@staticmethod
	def failed() -> StatusMessageType:
		return StatusMessageType("Failed")

	@staticmethod
	def skipped() -> StatusMessageType:
		return StatusMessageType("Skipped")

	@staticmethod
	def lost() -> StatusMessageType:
		return StatusMessageType("Lost")

	@staticmethod
	def removed() -> StatusMessageType:
		return StatusMessageType("Removed")

	@staticmethod
	def created() -> StatusMessageType:
		return StatusMessageType("Created")

	@staticmethod
	def updated() -> StatusMessageType:
		return StatusMessageType("Updated")

	@staticmethod
	def loaded() -> StatusMessageType:
		return StatusMessageType("Loaded")

	@staticmethod
	def imported() -> StatusMessageType:
		return StatusMessageType("Imported")

	@staticmethod
	def exported() -> StatusMessageType:
		return StatusMessageType("Exported")

	@staticmethod
	def sent() -> StatusMessageType:
		return StatusMessageType("Sent")

	@staticmethod
	def received() -> StatusMessageType:
		return StatusMessageType("Received")

	@staticmethod
	def done() -> StatusMessageType:
		return StatusMessageType("Done")

	@staticmethod
	def canceled() -> StatusMessageType:
		return StatusMessageType("Canceled")

	@staticmethod
	def rejected() -> StatusMessageType:
		return StatusMessageType("Rejected")

	@staticmethod
	def activated() -> StatusMessageType:
		return StatusMessageType("Activated")

	@staticmethod
	def deactivated() -> StatusMessageType:
		return StatusMessageType("Deactivated")

	@staticmethod
	def awaiting() -> StatusMessageType:
		return StatusMessageType("Awaiting")

	@staticmethod
	def locked() -> StatusMessageType:
		return StatusMessageType("Locked")

	@staticmethod
	def unlocked() -> StatusMessageType:
		return StatusMessageType("Unlocked")

	@staticmethod
	def saved() -> StatusMessageType:
		return StatusMessageType("Saved")

	@staticmethod
	def moved() -> StatusMessageType:
		return StatusMessageType("Moved")

	@staticmethod
	def copied() -> StatusMessageType:
		return StatusMessageType("Copied")

	@staticmethod
	def restored() -> StatusMessageType:
		return StatusMessageType("Restored")

	@staticmethod
	def checked() -> StatusMessageType:
		return StatusMessageType("Checked")

	@staticmethod
	def verified() -> StatusMessageType:
		return StatusMessageType("Verified")

	@staticmethod
	def approved() -> StatusMessageType:
		return StatusMessageType("Approved")

	@staticmethod
	def confirmed() -> StatusMessageType:
		return StatusMessageType("Confirmed")

	@staticmethod
	def changed() -> StatusMessageType:
		return StatusMessageType("Changed")

	@staticmethod
	def reloaded() -> StatusMessageType:
		return StatusMessageType("Reloaded")

	@staticmethod
	def launched() -> StatusMessageType:
		return StatusMessageType("Launched")

	@staticmethod
	def stopped() -> StatusMessageType:
		return StatusMessageType("Stopped")

	@staticmethod
	def suspended() -> StatusMessageType:
		return StatusMessageType("Suspended")

	@staticmethod
	def renewed() -> StatusMessageType:
		return StatusMessageType("Renewed")

	@staticmethod
	def included() -> StatusMessageType:
		return StatusMessageType("Included")

	@staticmethod
	def exclude() -> StatusMessageType:
		return StatusMessageType("Exclude")

	@staticmethod
	def installed() -> StatusMessageType:
		return StatusMessageType("Installed")

	@staticmethod
	def reset() -> StatusMessageType:
		return StatusMessageType("Reset")

	@staticmethod
	def connected() -> StatusMessageType:
		return StatusMessageType("Connected")

	@staticmethod
	def disconnected() -> StatusMessageType:
		return StatusMessageType("Disconnected")

	@staticmethod
	def enabled() -> StatusMessageType:
		return StatusMessageType("Enabled")

	@staticmethod
	def disabled() -> StatusMessageType:
		return StatusMessageType("Disabled")

	@staticmethod
	def inserted() -> StatusMessageType:
		return StatusMessageType("Inserted")

	@staticmethod
	def extracted() -> StatusMessageType:
		return StatusMessageType("Extracted")

	@staticmethod
	def open() -> StatusMessageType:
		return StatusMessageType("Open")

	@staticmethod
	def closed() -> StatusMessageType:
		return StatusMessageType("Closed")

	@staticmethod
	def empty() -> StatusMessageType:
		return StatusMessageType("...")

	@staticmethod
	def custom(custom_status_message: str) -> StatusMessageType:
		return StatusMessageType(custom_status_message)



"""
A module with an implementation of the base (parent) logger class.
\n
Copyright © 2023 Kalynovsky Valentin. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import datetime, platform, os, random

from mighty_logger.basic.patterns import Singleton
from mighty_logger.src.ansi_format import GetAnsiFormat
from mighty_logger.src.log_environment import LogEnvironments

class BasicLogger(Singleton):
	def __init__(
			self,
			program_name: str,
	) -> None:
		self._program_name = program_name
		self._settings: dict = {}
		self._ID = random.randint(1000000, 9999999)

	def _initialized_data(
			self,
			colors: list[str, str],
			env: str
	) -> str:
		"""
		A method that assemble an entry of system initialized data.

		:param colors: Color string list of initialized data
		:return: a string with initialized data
		"""
		if env == LogEnvironments.HTML:
			return (
					f"<span style='background-color: #{colors[1]};'>" +
					f"<span style='color: #{colors[0]};'>-{self._program_name}?entry> " +
					f"${platform.node()}^{os.getlogin()}" +
					f"@{platform.system()}" +
					f":{platform.version()}" +
					":{}:{}".format(*platform.architecture()) +
					f":{platform.machine()}" +
					f"</span></span>"
			)
		else:
			return (
					f"{colors[1]}" +
					f"{colors[0]}-{self._program_name}?entry> " +
					f"${platform.node()}^{os.getlogin()}" +
					f"@{platform.system()}" +
					f":{platform.version()}" +
					":{}:{}".format(*platform.architecture()) +
					f":{platform.machine()}" +
					f"{GetAnsiFormat('reset/on')}"
			)

	def _assemble_entry(
			self,
			colors: list[str, str, str, str, str, str],
			animation: list,
			icon: str,
			status_message_text: str,
			message_type: str,
			message_text: str,
			env: str,
			local_settings: dict
	) -> str:
		"""
		A method that assemble an entry into a string and returns it.

		:param colors: 6 colors that the method uses to assemble the string
		:param animation: Animation of entry
		:param icon: Type icon
		:param status_message_text: Status message
		:param message_type: Entry type
		:param message_text: Entry message
		:param env: For what environment is the string formed?
		:param local_settings: Settings for the string of the current entry
		:return: the formed entry string
		"""
		bold = local_settings['bold'] if 'bold' in local_settings else self._settings['global_bold_font']
		italic = local_settings['italic'] if 'italic' in local_settings else self._settings['global_italic_font']
		invert = local_settings['invert'] if 'invert' in local_settings else self._settings['global_invert_font']
		time_entry = local_settings['time_local_entry'] if 'time_local_entry' in local_settings else self._settings['time_global_entry']
		status_entry = local_settings['status_local_entry'] if 'status_local_entry' in local_settings else self._settings['status_global_entry']
		status_message_entry = local_settings['status_message_local_entry'] if 'status_message_local_entry' in local_settings else self._settings['status_message_global_entry']
		status_type_entry = local_settings['status_type_local_entry'] if 'status_type_local_entry' in local_settings else self._settings['status_type_global_entry']
		message_entry = local_settings['message_local_entry'] if 'message_local_entry' in local_settings else self._settings['message_global_entry']
		if env == LogEnvironments.HTML:
			return (
					(f"<b>" if bold else "") +
					(f"<i>" if italic else "") +
					f"<span style='background-color: #{colors[5]};'>" +
					f"<span style='color: #{colors[4]};'>-?entry> {animation[0]}</span>" +  # todo Must add animation
					(f"<span style='color: #{colors[0]};'>*{datetime.datetime.now()} </span>" if time_entry else "") +
					f"<div style='display: inline-block; white-space: pre; tab-size: 4'>{icon}&#9;</div>" +
					(f"<span style='color: #{colors[1]};'>#STATUS: </span>" if status_entry else "") +
					(f"<span style='color: #{colors[2]};'>{status_message_text} </span>" if status_message_entry else "") +
					(f"<span style='color: #{colors[3]};'>{message_type} - </span>" if status_type_entry else "") +
					(f"<span style='color: #{colors[4]};'>{message_text}</span></span>" if message_entry else "") +
					(f"</i>" if italic else "") +
					(f"</b>" if bold else "")
			)
		else:
			return (
					(f"{GetAnsiFormat('bold/on')}" if bold else "") +
					(f"{GetAnsiFormat('italic/on')}" if italic else "") +
					(f"{GetAnsiFormat('invert/on')}" if invert else "") +
					f"{colors[5]}" +
					f"{colors[4]}-?entry> {animation[0]}" +  # todo Must add animation
					(f"{colors[0]}*{datetime.datetime.now()} " if time_entry else "") +
					f"{icon}\t" +
					(f"{colors[1]}#STATUS: " if status_entry else "") +
					(f"{colors[2]}{status_message_text} " if status_message_entry else "") +
					(f"{colors[3]}{message_type} - " if status_type_entry else "") +
					(f"{colors[4]}{message_text}" if message_entry else "") +
					f"{GetAnsiFormat('reset/on')}"
			)
